@@ -3,6 +3,8 @@ from datetime import datetime, date, time, timedelta
 from decimal import Decimal, localcontext
 from dateutil.tz import tzoffset, tzutc
 import uuid
+import six
+from six.moves import map, reduce
 from .tds import *
 from .tds import _Column
 from .tdsproto import *
@@ -58,7 +60,7 @@ def make_param(tds, name, value):
         value = None
     if value is None:
         handler = DefaultHandler
-    elif isinstance(value, (int, long)):
+    elif isinstance(value, six.integer_types):
         if -2 ** 63 <= value <= 2 ** 63 - 1:
             handler = DefaultHandler
         elif -10 ** 38 + 1 <= value <= 10 ** 38 - 1:
@@ -70,7 +72,7 @@ def make_param(tds, name, value):
         handler = DefaultHandler
     elif isinstance(value, Binary):
         handler = DefaultHandler
-    elif isinstance(value, (str, unicode)):
+    elif isinstance(value, six.string_types):
         handler = DefaultHandler
     elif isinstance(value, datetime):
         if IS_TDS73_PLUS(tds):
@@ -151,16 +153,18 @@ class DefaultHandler(object):
         if size == -1:
             return None
 
-        chunk_handler = tds.chunk_handler
-        chunk_handler.begin(curcol, size)
         decoder = None
         if curcol.char_codec:
             decoder = curcol.char_codec.incrementaldecoder()
+            chunk_handler = MemoryStrChunkedHandler()
+        else:
+            chunk_handler = tds.chunk_handler
+        chunk_handler.begin(curcol, size)
         while True:
             chunk_len = r.get_int()
             if chunk_len <= 0:
                 if decoder:
-                    val = decoder.decode('', True)
+                    val = decoder.decode(b'', True)
                     chunk_handler.new_chunk(val)
                 return chunk_handler.end()
             left = chunk_len
@@ -278,7 +282,7 @@ class DefaultHandler(object):
             col.column_type = XSYBVARCHAR
             col.column_size = 1
             col.column_varint_size = tds_get_varint_size(tds, col.column_type)
-        elif isinstance(value, (int, long)):
+        elif isinstance(value, six.integer_types):
             col.column_type = SYBINTN
             if -2 ** 31 <= value <= 2 ** 31 - 1:
                 col.column_size = 4
@@ -303,7 +307,7 @@ class DefaultHandler(object):
                 col.column_type = XSYBVARBINARY
                 col.column_varint_size = tds_get_varint_size(tds, col.column_type)
             col.column_size = len(value)
-        elif isinstance(value, (str, unicode)):
+        elif isinstance(value, six.string_types):
             if len(value) > 4000:
                 if IS_TDS72_PLUS(tds):
                     col.column_type = XSYBNVARCHAR
@@ -553,6 +557,14 @@ def to_python(tds, data, type, length):
         raise Exception('unknown type {0}'.format(type))
 
 
+if sys.version_info[0] >= 3:
+    def _decode_num(buf):
+        return reduce(lambda acc, val: acc * 256 + val, reversed(buf), 0)
+else:
+    def _decode_num(buf):
+        return reduce(lambda acc, val: acc * 256 + ord(val), reversed(buf), 0)
+
+
 class NumericHandler(object):
     @staticmethod
     def get_info(tds, col):
@@ -566,7 +578,7 @@ class NumericHandler(object):
 
     @staticmethod
     def ms_parse_numeric(positive, buf, scale):
-        val = reduce(lambda acc, val: acc * 256 + ord(val), reversed(buf), 0)
+        val = _decode_num(buf)
         val = Decimal(val)
         with localcontext() as ctx:
             ctx.prec = 38
@@ -666,9 +678,9 @@ class NumericHandler(object):
             if not positive:
                 val *= -1
             size -= 1
-            val = long(val * (10 ** scale))
+            val = val * (10 ** scale)
         for i in range(size):
-            w.put_byte(val % 256)
+            w.put_byte(int(val % 256))
             val //= 256
         assert val == 0
 
@@ -818,7 +830,7 @@ XSYBVARCHAR	MS	2	0	1	1	0	0	0	1	-1	0
 
 # $Id: types.txt,v 1.5 2011/05/12 19:40:57 freddy77 Exp $
 '''
-    lines = map(lambda l: l.split('\t'), filter(lambda l: l and not l.startswith('#'), table.split('\n')))
+    lines = list(map(lambda l: l.split('\t'), filter(lambda l: l and not l.startswith('#'), table.split('\n'))))
     header = lines[0]
     data = lines[1:]
     hd = dict((b, a) for a, b in enumerate(header))
@@ -844,12 +856,12 @@ def tds_get_size_by_type(servertype):
 def tds_get_conversion_type(srctype, colsize):
     raise Exception('not implemented')
 '''
-    keyfunc = lambda (_, varint): varint
+    keyfunc = lambda val: val[1]
     code = fmt.format(
         '\n'.join('    if datatype in ({0},): return {1}'.format(','.join(name for name, _ in g), k) for k, g in groupby(sorted(gen_varint, key=keyfunc), keyfunc)),
         '\n'.join('        if datatype in ({0},): return {1}'.format(','.join(name for name, _ in g), k) for k, g in groupby(sorted(ms_varint, key=keyfunc), keyfunc)),
         '\n'.join('        if datatype in ({0},): return {1}'.format(','.join(name for name, _ in g), k) for k, g in groupby(sorted(syb_varint, key=keyfunc), keyfunc)),
-        '\n'.join('    if servertype in ({0},): return {1}'.format(','.join(name for name, _ in g), k) for k, g in groupby(sorted(types_sizes, key=lambda (_, size): size), lambda (_, size): size)),
+        '\n'.join('    if servertype in ({0},): return {1}'.format(','.join(name for name, _ in g), k) for k, g in groupby(sorted(types_sizes, key=keyfunc), keyfunc)),
         )
     return compile(code, 'autogenerated_types', 'exec')
 
@@ -887,10 +899,7 @@ def tds_fix_column_size(tds, curcol):
 
 
 def tds_convert_string(tds, char_codec, s):
-    if isinstance(s, bytes):
-        s = s.decode('utf8')
     return char_codec.encode(s)[0]
-    #return char_codec.encode(s)[0]
 
 _utc = tzutc()
 
@@ -926,7 +935,7 @@ class MsDatetimeHandler(object):
             if size < 3 or size > 5:
                 raise Exception('TDS_FAIL')
             time_buf = readall(r, size)
-            val = reduce(lambda acc, val: acc * 256 + ord(val), reversed(time_buf), 0)
+            val = _decode_num(time_buf)
             val *= 10 ** (7 - col.prec)
             nanoseconds = val * 100
 
@@ -934,7 +943,7 @@ class MsDatetimeHandler(object):
         days = 0
         if col.column_type != SYBMSTIME:
             date_buf = readall(r, 3)
-            days = reduce(lambda acc, val: acc * 256 + ord(val), reversed(date_buf), 0)
+            days = _decode_num(date_buf)
 
         # get time offset
         if col.column_type == SYBMSTIME:
@@ -1039,7 +1048,7 @@ class MsDatetimeHandler(object):
         if col.column_type == SYBMSDATETIMEOFFSET:
             # Encoding timezone part
             assert utcoffset is not None
-            parts.append(struct.pack('<h', utcoffset.total_seconds() // 60))
+            parts.append(struct.pack('<h', int(utcoffset.total_seconds()) // 60))
         size = reduce(lambda a, b: a + len(b), parts, 0)
         w.put_byte(size)
         for part in parts:
