@@ -1,10 +1,10 @@
 """DB-SIG compliant module for communicating with MS SQL servers"""
 
 __author__ = 'Mikhail Denisenko <denisenkom@gmail.com>'
-__version__ = '1.3.6'
+__version__ = '1.4.0'
 
 import logging
-import errno
+import six
 from . import lcid
 from .tds import *
 from .login import *
@@ -56,12 +56,36 @@ class _Connection(object):
         """
         return self._autocommit
 
+    @property
+    def autocommit(self):
+        return self._autocommit
+
+    @autocommit.setter
+    def autocommit(self, value):
+        if self._autocommit != value:
+            if value:
+                if self._conn.tds72_transaction:
+                    self._try_activate_cursor(None)
+                    self._cancel(self._conn.main_session)
+                    tds_submit_commit(self._conn.main_session, False)
+                    self._sqlok(self._conn.main_session)
+            else:
+                self._cancel(self._conn.main_session)
+                tds_submit_begin_tran(self._conn.main_session)
+                self._sqlok(self._conn.main_session)
+            self._autocommit = value
+
     def _assert_open(self):
         if not self._conn:
             raise Error('Connection closed')
         if not self._conn.is_connected():
             self._open()
 
+    def _trancount(self):
+        with self.cursor() as cur:
+            cur.execute('select @@trancount')
+            return cur.fetchone()[0]
+
     @property
     def chunk_handler(self):
         '''
@@ -99,48 +123,14 @@ class _Connection(object):
     def _open(self):
         self._state = DB_RES_NO_MORE_RESULTS
         self._active_cursor = None
-        from tds import _TdsSocket
+        from .tds import _TdsSocket
         self._conn = None
-        self._dirty = False
-        login = self._login
-        if IS_TDS7_PLUS(login) and login.instance_name and not login.port:
-            instances = tds7_get_instances(login.server_name)
-            if login.instance_name not in instances:
-                raise LoginError("Instance {0} not found on server {1}".format(login.instance_name, login.server_name))
-            instdict = instances[login.instance_name]
-            if 'tcp' not in instdict:
-                raise LoginError("Instance {0} doen't have tcp connections enabled".format(login.instance_name))
-            login.port = int(instdict['tcp'])
-        if not login.port:
-            login.port = 1433
-        connect_timeout = login.connect_timeout
-        login.query_timeout = login.connect_timeout if login.connect_timeout else login.query_timeout
-        for host in login.load_balancer.choose():
-            try:
-                sock = socket.create_connection(
-                    (host, login.port),
-                    connect_timeout or 90000)
-                sock.settimeout(login.query_timeout)
-                sock.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
-            except socket.error as e:
-                err = LoginError("Cannot connect to server '{0}': {1}".format(host, e), e)
-                continue
-            try:
-                self._conn = _TdsSocket(self._login, sock)
-                break
-            except Exception as e:
-                sock.close()
-                err = e
-                #raise
-                continue
-        else:
-            raise err
-        self._active_cursor = self._main_cursor = self.cursor()
+        self._conn = _TdsSocket(self._login)
         if not self._autocommit:
             tds_submit_begin_tran(self._conn.main_session)
-        self._sqlok(self._conn.main_session)
+            self._sqlok(self._conn.main_session)
 
-    def __init__(self, login, as_dict, autocommit):
+    def __init__(self, login, as_dict, autocommit=False):
         self._autocommit = autocommit
         self._login = login
         self._as_dict = as_dict
@@ -170,7 +160,6 @@ class _Connection(object):
         self._sqlok(conn.main_session)
         while self._nextset(conn.main_session):
             pass
-        self._dirty = False
 
     def cursor(self):
         """
@@ -191,6 +180,9 @@ class _Connection(object):
             if not self._conn or not self._conn.is_connected():
                 return
 
+            if not self._conn.tds72_transaction:
+                return
+
             session = self._conn.main_session
             self._cancel(session)
             self._active_cursor = None
@@ -198,12 +190,6 @@ class _Connection(object):
             self._sqlok(session)
             while self._nextset(session):
                 pass
-            self._dirty = False
-        except socket.error as e:
-            if e.errno == errno.ECONNRESET:
-                return
-        except ClosedConnectionError:
-            pass
         except:
             logger.exception('unexpected error in rollback')
 
@@ -219,7 +205,7 @@ class _Connection(object):
         It can be called more than once in a row. No exception is raised in
         this case.
         """
-        logger.debug("MSSQLConnection._cancel()")
+        #logger.debug("MSSQLConnection._cancel()")
         tds_send_cancel(session)
         tds_process_cancel(session)
 
@@ -231,7 +217,7 @@ class _Connection(object):
         It can be called more than once in a row. No exception is raised in
         this case.
         """
-        logger.debug("MSSQLConnection.close()")
+        #logger.debug("MSSQLConnection.close()")
         self._assert_open()
         self._conn.close()
         self._conn = None
@@ -243,7 +229,7 @@ class _Connection(object):
         This function selects the given database. An exception is raised on
         failure.
         """
-        logger.debug("MSSQLConnection.select_db()")
+        #logger.debug("MSSQLConnection.select_db()")
         self._assert_open()
         cur = self.cursor()
         try:
@@ -254,7 +240,7 @@ class _Connection(object):
     _nextrow_mask = TDS_STOPAT_ROWFMT | TDS_RETURN_DONE | TDS_RETURN_ROW | TDS_RETURN_COMPUTE
 
     def _nextrow(self, session):
-        logger.debug("_nextrow()")
+        #logger.debug("_nextrow()")
         resinfo = session.res_info
         if not resinfo or self._state != DB_RES_RESULTSET_ROWS:
             # no result set or result set empty (no rows)
@@ -280,7 +266,7 @@ class _Connection(object):
             raise Exception("unexpected result from tds_process_tokens")
 
     def _sqlok(self, session):
-        logger.debug("dbsqlok()")
+        #logger.debug("dbsqlok()")
         #CHECK_CONN(FAIL);
 
         #
@@ -288,7 +274,7 @@ class _Connection(object):
         # submitted returned no data (like an insert) -- then
         # we process the end token to extract the status code.
         #
-        logger.debug("dbsqlok() not done, calling tds_process_tokens()")
+        #logger.debug("dbsqlok() not done, calling tds_process_tokens()")
         while True:
             tds_code, result_type, done_flags = tds_process_tokens(session, TDS_TOKEN_RESULTS)
 
@@ -341,9 +327,7 @@ class _Connection(object):
         cols = session.res_info.columns
         row = tuple(col.value for col in cols)
         if self.as_dict:
-            row_dict = dict(enumerate(cols))
-            row_dict.update(dict((col.column_name, col.value) for col in cols if col.column_name))
-            row = row_dict
+            row = dict((col.column_name, col.value) for col in cols if col.column_name)
         return row
 
     def _nextset(self, session):
@@ -379,7 +363,7 @@ class _Connection(object):
         else:
             return None
 
-    def _native_description(self):
+    def _native_description(self, cursor):
         session = cursor._session
         if session is None:
             return None
@@ -455,26 +439,8 @@ class _Connection(object):
             else:
                 logger.error('logic error: tds_process_tokens result_type %d', result_type)
 
-    def _exec_with_retry(self, fun):
-        self._assert_open()
-        in_tran = self._conn.tds72_transaction
-        if in_tran and self._dirty:
-            self._dirty = True
-            return fun()
-        else:
-            # first attemp
-            try:
-                self._dirty = True
-                return fun()
-            except socket.error as e:
-                if e.errno != errno.ECONNRESET:
-                    raise
-            # try again if connection was reset
-            self._assert_open()
-            return fun()
-
     def _callproc(self, cursor, procname, parameters):
-        logger.debug('callproc begin')
+        #logger.debug('callproc begin')
         self._assert_open()
         self._try_activate_cursor(cursor)
         session = cursor._session
@@ -513,7 +479,7 @@ class _Connection(object):
                 break
             else:
                 logger.error('logic error: tds_process_tokens result_type %d', result_type)
-        logger.debug('callproc end')
+        #logger.debug('callproc end')
         results = list(parameters)
         for key, param in session.output_params.items():
             results[key] = param.value
@@ -523,7 +489,7 @@ class _Connection(object):
 ##################
 ## Cursor class ##
 ##################
-class _Cursor(object):
+class _Cursor(six.Iterator):
     """
     This class represents a database cursor, which is used to issue queries
     and fetch results from a database connection.
@@ -536,20 +502,7 @@ class _Cursor(object):
 
     def _open(self):
         if self._conn._conn.mars_enabled:
-            in_tran = self._conn._conn.tds72_transaction
-            if in_tran and self._conn._dirty:
-                self._session = self._conn._conn.create_session()
-            else:
-                try:
-                    self._session = self._conn._conn.create_session()
-                    return
-                except socket.error as e:
-                    if e.errno != errno.ECONNRESET:
-                        raise
-                except ClosedConnectionError:
-                    pass
-                self._conn._assert_open()
-                self._session = self._conn._conn.create_session()
+            self._session = self._conn._conn.create_session()
         else:
             self._session = self._conn._conn.main_session
 
@@ -579,7 +532,7 @@ class _Cursor(object):
         :keyword parameters: The optional parameters for the procedure
         :type parameters: sequence
         """
-        return self._conn._exec_with_retry(lambda: self._conn._callproc(self, procname, parameters))
+        return self._conn._callproc(self, procname, parameters)
 
     @property
     def return_value(self):
@@ -589,14 +542,6 @@ class _Cursor(object):
     def connection(self):
         return self._conn
 
-    @property
-    def spid(self):
-        # not thread safe for connection
-        dirty = self._conn._dirty
-        spid = self.execute_scalar('select @@SPID')
-        self._conn._dirty = dirty
-        return spid
-
     def get_proc_return_status(self):
         return self._conn._get_proc_return_status(self)
 
@@ -610,7 +555,7 @@ class _Cursor(object):
         if self._conn is not None:
             self._conn._close_cursor(self)
 
-    def _execute(self, operation, params):
+    def execute(self, operation, params=()):
         # Execute the query
         if params:
             if isinstance(params, (list, tuple)):
@@ -625,13 +570,10 @@ class _Cursor(object):
                 rename = dict((name, '@{0}'.format(name)) for name in params.keys())
                 params = dict(('@{0}'.format(name), value) for name, value in params.items())
                 operation = operation % rename
-            logger.debug('converted query: {0}'.format(operation))
-            logger.debug('params: {0}'.format(params))
+            #logger.debug('converted query: {0}'.format(operation))
+            #logger.debug('params: {0}'.format(params))
         self._conn._execute(self, operation, params)
 
-    def execute(self, operation, params=()):
-        self._conn._exec_with_retry(lambda: self._execute(operation, params))
-
     def executemany(self, operation, params_seq):
         counts = []
         for params in params_seq:
@@ -701,7 +643,7 @@ class _Cursor(object):
     def fetchall(self):
         return list(row for row in self)
 
-    def next(self):
+    def __next__(self):
         row = self.fetchone()
         if row is None:
             raise StopIteration
@@ -723,7 +665,7 @@ class _Cursor(object):
 def connect(server='.', database='', user='', password='', timeout=0,
             login_timeout=60, as_dict=False,
             host='', appname=None, port=None, tds_version=TDS74,
-            encryption_level=TDS_ENCRYPTION_OFF, autocommit=True,
+            encryption_level=TDS_ENCRYPTION_OFF, autocommit=False,
             blocksize=4096, use_mars=False, auth=None, readonly=False,
             load_balancer=None, use_tz=None):
     """
@@ -749,6 +691,7 @@ def connect(server='.', database='', user='', password='', timeout=0,
     :keyword port: the TCP port to use to connect to the server
     :type appname: string
     """
+
     # set the login timeout
     try:
         login_timeout = int(login_timeout)
