@@ -1,7 +1,8 @@
 import logging
 import traceback
 from .tdsproto import *
-from .tds import _Column
+from .tds import _Column, _Results
+from . import tds as tdsflags
 from .data import *
 
 logger = logging.getLogger(__name__)
@@ -137,14 +138,14 @@ def tds_process_default_tokens(tds, marker):
     elif marker == TDS_CURINFO_TOKEN:
         return tds_process_cursor_tokens(tds)
     elif marker in (TDS5_DYNAMIC_TOKEN, TDS_LOGINACK_TOKEN, TDS_ORDERBY_TOKEN, TDS_CONTROL_TOKEN):
-        #logger.debug("Eating %s token", tds_token_name(marker))
+        #logger.warning("Eating %s token", tds_token_name(marker))
         r.skip(r.get_smallint())
     elif marker == TDS_TABNAME_TOKEN:  # used for FOR BROWSE query
         return tds_process_tabname(tds)
     elif marker == TDS_COLINFO_TOKEN:
         return tds_process_colinfo(tds, None, 0)
     elif marker == TDS_ORDERBY2_TOKEN:
-        #logger.debug("Eating %s token", tds_token_name(marker))
+        #logger.warning("Eating %s token", tds_token_name(marker))
         r.skip(r.get_int())
     elif marker == TDS_NBC_ROW_TOKEN:
         return tds_process_nbcrow(tds)
@@ -170,6 +171,14 @@ def tds_process_row(tds):
     return TDS_SUCCESS
 
 
+if sys.version_info[0] >= 3:
+    def _ord(val):
+        return val
+else:
+    def _ord(val):
+        return ord(val)
+
+
 # NBC=null bitmap compression row
 # http://msdn.microsoft.com/en-us/library/dd304783(v=prot.20).aspx
 def tds_process_nbcrow(tds):
@@ -182,9 +191,9 @@ def tds_process_nbcrow(tds):
 
     # reading bitarray for nulls, 1 represent null values for
     # corresponding fields
-    nbc = readall(r, (len(info.columns) + 7) / 8)
+    nbc = readall(r, (len(info.columns) + 7) // 8)
     for i, curcol in enumerate(info.columns):
-        if ord(nbc[i / 8]) & (1 << (i % 8)):
+        if _ord(nbc[i // 8]) & (1 << (i % 8)):
             curcol.value = None
         else:
             curcol.value = curcol.funcs.get_data(tds, curcol)
@@ -370,11 +379,11 @@ _SERVER_TO_CLIENT_MAPPING = {
 def tds_process_login_tokens(tds):
     r = tds._reader
     succeed = False
-    #logger.debug('tds_process_login_tokens()')
+    logger.debug('tds_process_login_tokens()')
     ver = {}
     while True:
         marker = r.get_byte()
-        #logger.debug('looking for login token, got  {0:x}({1})'.format(marker, tds_token_name(marker)))
+        logger.debug('looking for login token, got  {0:x}({1})'.format(marker, tds_token_name(marker)))
         if marker == TDS_LOGINACK_TOKEN:
             tds.tds71rev1 = 0
             size = r.get_smallint()
@@ -400,7 +409,7 @@ def tds_process_login_tokens(tds):
                 ver['name'] = '2012'
             else:
                 ver['name'] = 'unknown'
-            #logger.debug('server reports TDS version {0:x}'.format(version))
+            logger.debug('server reports TDS version {0:x}'.format(version))
             # get server product name
             # ignore product name length, some servers seem to set it incorrectly
             r.get_byte()
@@ -408,7 +417,7 @@ def tds_process_login_tokens(tds):
             size -= 10
             if IS_TDS7_PLUS(tds):
                 product_version = 0x80000000
-                tds.conn.product_name = r.read_ucs2(size / 2)
+                tds.conn.product_name = r.read_ucs2(size // 2)
             elif IS_TDS5_PLUS(tds):
                 raise NotImplementedError()
                 #tds.product_name = tds_get_string(tds, size)
@@ -419,7 +428,7 @@ def tds_process_login_tokens(tds):
             # MSSQL 6.5 and 7.0 seem to return strange values for this
             # using TDS 4.2, something like 5F 06 32 FF for 6.50
             tds.conn.product_version = product_version
-            #logger.debug('Product version {0:x}'.format(product_version))
+            logger.debug('Product version {0:x}'.format(product_version))
             # TDS 5.0 reports 5 on success 6 on failure
             # TDS 4.2 reports 1 on success and is not present of failure
             if ack == 5 or ack == 1:
@@ -493,13 +502,10 @@ def tds_process_tokens(tds, flag):
     done_flags = 0
     saved_rows_affected = tds.rows_affected
     cancel_seen = 0
-    import tds as tdsflags
     r = tds._reader
 
-    def SET_RETURN(ret, f):
+    def SET_RETURN(ret, return_flag, stopat_flag):
         parent['result_type'] = ret
-        return_flag = getattr(tdsflags, 'TDS_RETURN_' + f)
-        stopat_flag = getattr(tdsflags, 'TDS_STOPAT_' + f)
         parent['return_flag'] = return_flag | stopat_flag
         if flag & stopat_flag:
             r.unget_byte()
@@ -530,7 +536,7 @@ def tds_process_tokens(tds, flag):
                     else:
                         rc = tds_process_tabname(tds)
                 else:
-                    if SET_RETURN(TDS_ROWFMT_RESULT, 'ROWFMT'):
+                    if SET_RETURN(TDS_ROWFMT_RESULT, tdsflags.TDS_RETURN_ROWFMT, tdsflags.TDS_STOPAT_ROWFMT):
                         rc = tds7_process_result(tds)
                         # handle browse information (if presents)
                         marker = r.get_byte()
@@ -540,15 +546,15 @@ def tds_process_tokens(tds, flag):
                         else:
                             rc = tds_process_tabname(tds)
             elif marker == TDS_RESULT_TOKEN:
-                if SET_RETURN(TDS_ROWFMT_RESULT, 'ROWFMT'):
+                if SET_RETURN(TDS_ROWFMT_RESULT, tdsflags.TDS_RETURN_ROWFMT, tdsflags.TDS_STOPAT_ROWFMT):
                     rc = tds_process_result(tds)
             elif marker == TDS_ROWFMT2_TOKEN:
-                if SET_RETURN(TDS_ROWFMT_RESULT, 'ROWFMT'):
+                if SET_RETURN(TDS_ROWFMT_RESULT, tdsflags.TDS_RETURN_ROWFMT, tdsflags.TDS_STOPAT_ROWFMT):
                     rc = tds5_process_result(tds)
             elif marker == TDS_COLNAME_TOKEN:
                 rc = tds_process_col_name(tds)
             elif marker == TDS_COLFMT_TOKEN:
-                if SET_RETURN(TDS_ROWFMT_RESULT, 'ROWFMT'):
+                if SET_RETURN(TDS_ROWFMT_RESULT, tdsflags.TDS_RETURN_ROWFMT, tdsflags.TDS_STOPAT_ROWFMT):
                     rc = tds_process_col_fmt(tds)
                     # handle browse information (if present)
                     marker = r.get_byte()
@@ -581,15 +587,15 @@ def tds_process_tokens(tds, flag):
                                 and tds.cur_dyn and tds.cur_dyn.num_id == 0 and curcol.value:
                             tds.cur_dyn.num_id = curcol.value
                 else:
-                    if SET_RETURN(TDS_PARAM_RESULT, 'PROC'):
+                    if SET_RETURN(TDS_PARAM_RESULT, tdsflags.TDS_RETURN_PROC, tdsflags.TDS_STOPAT_PROC):
                         rc = tds_process_param_result_tokens(tds)
             elif marker == TDS_COMPUTE_NAMES_TOKEN:
                 rc = tds_process_compute_names(tds)
             elif marker == TDS_COMPUTE_RESULT_TOKEN:
-                if SET_RETURN(TDS_COMPUTEFMT_RESULT, 'COMPUTEFMT'):
+                if SET_RETURN(TDS_COMPUTEFMT_RESULT, tdsflags.TDS_RETURN_COMPUTEFMT, tdsflags.TDS_STOPAT_COMPUTEFMT):
                     rc = tds_process_compute_result(tds)
             elif marker == TDS7_COMPUTE_RESULT_TOKEN:
-                if SET_RETURN(TDS_COMPUTEFMT_RESULT, 'COMPUTEFMT'):
+                if SET_RETURN(TDS_COMPUTEFMT_RESULT, tdsflags.TDS_RETURN_COMPUTEFMT, tdsflags.TDS_STOPAT_COMPUTEFMT):
                     rc = tds7_process_compute_result(tds)
             elif marker in (TDS_ROW_TOKEN, TDS_NBC_ROW_TOKEN):
                 # overstepped the mark...
@@ -605,7 +611,7 @@ def tds_process_tokens(tds, flag):
                 # I don't know when this it's false but it happened, also server can send garbage...
                 if tds.current_results:
                     tds.current_results.rows_exist = 1
-                if SET_RETURN(TDS_ROW_RESULT, 'ROW'):
+                if SET_RETURN(TDS_ROW_RESULT, tdsflags.TDS_RETURN_ROW, tdsflags.TDS_STOPAT_ROW):
                     if marker == TDS_NBC_ROW_TOKEN:
                         rc = tds_process_nbcrow(tds)
                     else:
@@ -614,7 +620,7 @@ def tds_process_tokens(tds, flag):
                 # I don't know when this it's false but it happened, also server can send garbage...
                 if tds.res_info:
                     tds.res_info.rows_exist = 1
-                if SET_RETURN(TDS_COMPUTE_RESULT, 'COMPUTE'):
+                if SET_RETURN(TDS_COMPUTE_RESULT, tdsflags.TDS_RETURN_COMPUTE, tdsflags.TDS_STOPAT_COMPUTE):
                     rc = tds_process_compute(tds, NULL)
             elif marker == TDS_RETURNSTATUS_TOKEN:
                 ret_status = r.get_int()
@@ -626,7 +632,7 @@ def tds_process_tokens(tds, flag):
                     else:
                         # TODO optimize
                         flag &= ~TDS_STOPAT_PROC
-                        if SET_RETURN(TDS_STATUS_RESULT, 'PROC'):
+                        if SET_RETURN(TDS_STATUS_RESULT, tdsflags.TDS_RETURN_PROC, tdsflags.TDS_STOPAT_PROC):
                             tds.has_status = True
                             tds.ret_status = ret_status
                             #logger.debug("tds_process_tokens: return status is {0}".format(tds.ret_status))
@@ -645,27 +651,27 @@ def tds_process_tokens(tds, flag):
                                 rc, done_flags = tds_process_end(tds, marker)
                                 done_flags &= ~TDS_DONE_ERROR
                                 # FIXME warning to macro expansion
-                                SET_RETURN(TDS_DONE_RESULT, 'DONE')
+                                SET_RETURN(TDS_DONE_RESULT, tdsflags.TDS_RETURN_DONE, tdsflags.TDS_STOPAT_DONE)
                             else:
                                 r.unget_byte()
                     else:
                         r.unget_byte()
             elif marker == TDS5_PARAMFMT_TOKEN:
-                if SET_RETURN(TDS_DESCRIBE_RESULT, 'PARAMFMT'):
+                if SET_RETURN(TDS_DESCRIBE_RESULT, tdsflags.TDS_RETURN_PARAMFMT, tdsflags.TDS_STOPAT_PARAMFMT):
                     rc = tds_process_dyn_result(tds)
             elif marker == TDS5_PARAMFMT2_TOKEN:
-                if SET_RETURN(TDS_DESCRIBE_RESULT, 'PARAMFMT'):
+                if SET_RETURN(TDS_DESCRIBE_RESULT, tdsflags.TDS_RETURN_PARAMFMT, tdsflags.TDS_STOPAT_PARAMFMT):
                     rc = tds5_process_dyn_result2(tds)
             elif marker == TDS5_PARAMS_TOKEN:
-                if SET_RETURN(TDS_PARAM_RESULT, 'PROC'):
+                if SET_RETURN(TDS_PARAM_RESULT, tdsflags.TDS_RETURN_PROC, tdsflags.TDS_STOPAT_PROC):
                     rc = tds_process_params_result_token(tds)
             elif marker == TDS_CURINFO_TOKEN:
                 rc = tds_process_cursor_tokens(tds)
             elif marker == TDS_DONE_TOKEN:
-                if SET_RETURN(TDS_DONE_RESULT, 'DONE'):
+                if SET_RETURN(TDS_DONE_RESULT, tdsflags.TDS_RETURN_DONE, tdsflags.TDS_STOPAT_DONE):
                     rc, done_flags = tds_process_end(tds, marker)
             elif marker == TDS_DONEPROC_TOKEN:
-                if SET_RETURN(TDS_DONEPROC_RESULT, 'DONE'):
+                if SET_RETURN(TDS_DONEPROC_RESULT, tdsflags.TDS_RETURN_DONE, tdsflags.TDS_STOPAT_DONE):
                     rc, done_flags = tds_process_end(tds, marker)
                     if tds.internal_sp_called in (0, TDS_SP_PREPARE,
                                                   TDS_SP_PREPEXEC, TDS_SP_EXECUTE,
@@ -694,13 +700,13 @@ def tds_process_tokens(tds, flag):
                     if tds.rows_affected != TDS_NO_COUNT:
                         saved_rows_affected = tds.rows_affected
                 else:
-                    if SET_RETURN(TDS_DONEINPROC_RESULT, 'DONE'):
+                    if SET_RETURN(TDS_DONEINPROC_RESULT, tdsflags.TDS_RETURN_DONE, tdsflags.TDS_STOPAT_DONE):
                         rc, done_flags = tds_process_end(tds, marker)
             elif marker in (TDS_ERROR_TOKEN, TDS_INFO_TOKEN, TDS_EED_TOKEN):
-                if SET_RETURN(TDS_MSG_RESULT, 'MSG'):
+                if SET_RETURN(TDS_MSG_RESULT, tdsflags.TDS_RETURN_MSG, tdsflags.TDS_STOPAT_MSG):
                     rc = tds_process_default_tokens(tds, marker)
             else:
-                if SET_RETURN(TDS_OTHERS_RESULT, 'OTHERS'):
+                if SET_RETURN(TDS_OTHERS_RESULT, tdsflags.TDS_RETURN_OTHERS, tdsflags.TDS_STOPAT_OTHERS):
                     rc = tds_process_default_tokens(tds, marker)
 
             cancel_seen |= tds.in_cancel
@@ -744,25 +750,6 @@ def tds_process_cancel(tds):
             return TDS_SUCCESS
 
 
-def get_api_coltype(coltype):
-    if coltype in (SYBBIT, SYBINT1, SYBINT2, SYBINT4, SYBINT8, SYBINTN,
-                   SYBREAL, SYBFLT8, SYBFLTN):
-        return NUMBER
-    elif coltype in (SYBMONEY, SYBMONEY4, SYBMONEYN, SYBNUMERIC,
-                     SYBDECIMAL):
-        return DECIMAL
-    elif coltype in (SYBDATETIME, SYBDATETIME4, SYBDATETIMN):
-        return DATETIME
-    elif coltype in (SYBVARCHAR, SYBCHAR, SYBTEXT,
-                     XSYBNVARCHAR, XSYBNCHAR, SYBNTEXT,
-                     XSYBVARCHAR, XSYBCHAR):
-        return STRING
-    else:
-        return BINARY
-
-from tds import _Results
-
-
 #/**
 # * tds7_process_result() is the TDS 7.0 result set processing routine.  It
 # * is responsible for populating the tds->res_info structure.
@@ -807,12 +794,14 @@ def tds7_process_result(tds):
         curcol = _Column()
         info.columns.append(curcol)
         tds7_get_data_info(tds, curcol)
-        coltype = get_api_coltype(curcol.column_type)
+        coltype = curcol.column_type
+        if coltype == SYBINTN:
+            coltype = {1: SYBINT1, 2: SYBINT2, 4: SYBINT4, 8: SYBINT8}[curcol.column_size]
         precision = curcol.column_prec if hasattr(curcol, 'column_prec') else None
         scale = curcol.column_scale if hasattr(curcol, 'column_scale') else None
         header_tuple.append((curcol.column_name, coltype, None, None, precision, scale, curcol.column_nullable))
-    info.native_descr = tuple((col.column_name, col.column_type)
-                              for col in tds.res_info.columns)
+    info.native_descr = tuple((col[0], col[1])
+                              for col in header_tuple)
     info.description = tuple(header_tuple)
     return info
 
